@@ -37,6 +37,7 @@
 #include "bgpd/bgp_mplsvpn.h"
 #include "bgpd/bgp_nexthop.h"
 #include "bgpd/bgp_vty.h"
+#include "bgpd/bgp_network.h"
 
 #define VT100_RESET "\x1b[0m"
 #define VT100_RED "\x1b[31m"
@@ -48,7 +49,7 @@
 #define OPT_PARAM  2
 
 /* need these to link in libbgp */
-struct zebra_privs_t *bgpd_privs = NULL;
+struct zebra_privs_t bgpd_privs = {};
 struct thread_master *master = NULL;
 
 static int failed = 0;
@@ -945,6 +946,31 @@ static struct test_segment mp_unreach_segments[] = {
 	},
 	{NULL, NULL, {0}, 0, 0}};
 
+static struct test_segment mp_prefix_sid[] = {
+	{
+		"PREFIX-SID",
+		"PREFIX-SID Test 1",
+		{
+			/* TLV[0] Latel-Index TLV */
+			0x01,                    /* Type 0x01:Label-Index */
+			0x00, 0x07,              /* Length */
+			0x00,                    /* RESERVED */
+			0x00, 0x00,              /* Flags */
+			0x00, 0x00, 0x00, 0x02,  /* Label Index */
+
+			/* TLV[1] SRGB TLV */
+			0x03,                    /* Type 0x03:SRGB */
+			0x00, 0x08,              /* Length */
+			0x00, 0x00,              /* Flags */
+			0x0a, 0x1b, 0xfe,        /* SRGB[0] first label */
+			0x00, 0x00, 0x0a         /* SRBG[0] nb-labels in range */
+		},
+		.len = 21,
+		.parses = SHOULD_PARSE,
+	},
+	{NULL, NULL, { 0 }, 0, 0},
+};
+
 /* nlri_parse indicates 0 on successful parse, and -1 otherwise.
  * attr_parse indicates BGP_ATTR_PARSE_PROCEED/0 on success,
  * and BGP_ATTR_PARSE_ERROR/-1 or lower negative ret on err.
@@ -1000,10 +1026,20 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 
 	printf("%s: %s\n", t->name, t->desc);
 
-	if (type == BGP_ATTR_MP_REACH_NLRI)
+	switch (type) {
+	case BGP_ATTR_MP_REACH_NLRI:
 		parse_ret = bgp_mp_reach_parse(&attr_args, &nlri);
-	else
+		break;
+	case BGP_ATTR_MP_UNREACH_NLRI:
 		parse_ret = bgp_mp_unreach_parse(&attr_args, &nlri);
+		break;
+	case BGP_ATTR_PREFIX_SID:
+		parse_ret = bgp_attr_prefix_sid(&attr_args);
+		break;
+	default:
+		printf("unknown type");
+		return;
+	}
 	if (!parse_ret) {
 		iana_afi_t pkt_afi;
 		iana_safi_t pkt_safi;
@@ -1022,7 +1058,7 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 	if (!parse_ret) {
 		if (type == BGP_ATTR_MP_REACH_NLRI)
 			nlri_ret = bgp_nlri_parse(peer, &attr, &nlri, 0);
-		else
+		else if (type == BGP_ATTR_MP_UNREACH_NLRI)
 			nlri_ret = bgp_nlri_parse(peer, &attr, &nlri, 1);
 	}
 	handle_result(peer, t, parse_ret, nlri_ret);
@@ -1050,7 +1086,7 @@ int main(void)
 	cmd_init(0);
 	bgp_vty_init();
 	master = thread_master_create("test mp attr");
-	bgp_master_init(master);
+	bgp_master_init(master, BGP_SOCKET_SNDBUF_SIZE, list_new());
 	vrf_init(NULL, NULL, NULL, NULL);
 	bgp_option_set(BGP_OPT_NO_LISTEN);
 	bgp_attr_init();
@@ -1058,7 +1094,7 @@ int main(void)
 	if (fileno(stdout) >= 0)
 		tty = isatty(fileno(stdout));
 
-	if (bgp_get(&bgp, &asn, NULL, BGP_INSTANCE_TYPE_DEFAULT))
+	if (bgp_get(&bgp, &asn, NULL, BGP_INSTANCE_TYPE_DEFAULT) < 0)
 		return -1;
 
 	peer = peer_create_accept(bgp);
@@ -1085,6 +1121,10 @@ int main(void)
 		parse_test(peer, &mp_unreach_segments[i++],
 			   BGP_ATTR_MP_UNREACH_NLRI);
 
+	i = 0;
+	while (mp_prefix_sid[i].name)
+		parse_test(peer, &mp_prefix_sid[i++],
+			   BGP_ATTR_PREFIX_SID);
 	printf("failures: %d\n", failed);
 	return failed;
 }
